/**
  *  \file int/export.h
  *  \brief Interpreter: Export
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_EXPORT_H
#define PCC_V2_INT_EXPORT_H

#include <vector>
#include "cpluslib/string.h"
#include "int/if/ifutil.h"

class IntContext;
class AnswerProvider;
class TextFile;

/** Export field list. This defines the user's setup of an export layout. */
class IntExportFieldList {
    struct Item {
        string_t name;
        int width;
        Item(const string_t& name, int width)
            : name(name),
              width(width)
            { }
    };
    std::vector<Item> items;

 public:
    typedef std::vector<Item>::size_type index_t;
    
    IntExportFieldList();
    ~IntExportFieldList();

    void addList(string_t spec);
    void add(string_t spec);
    void add(index_t line, string_t name, int width);
    void swap(index_t a, index_t b);
    void remove(index_t line);

    void setFieldName(index_t line, string_t name);
    void setFieldWidth(index_t line, int width);
    const string_t& getFieldName(index_t line) const;
    const int getFieldWidth(index_t line) const;
    const index_t size() const;

    string_t toString();
};

/** Exporter base. This class contains the logic of obtaining export
    data. Actual output is provided by derived classes. */
class IntExporter {
 protected:
    virtual void startTable(const IntExportFieldList& fields, const TIntTypeHint types[]) = 0;
    virtual void startRecord() = 0;
    virtual void addField(IntValue* value, const string_t& name, TIntTypeHint type) = 0;
    virtual void endRecord() = 0;
    virtual void endTable() = 0;

 public:
    IntExporter();
    virtual ~IntExporter();

    void doExport(IntContext* ctx,
                  AnswerProvider& filter,
                  const IntExportFieldList& fields);
};

#endif
