/**
  *  \file int/subr.cc
  *  \brief Interpreter: Subroutine Value
  *
  *  This used to be in int/compound.cc, but has been moved into a separate
  *  module to reduce link dependencies. c2talk, c2mailout get 300k smaller
  *  when this is separate.
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "int/compound.h"
#include "int/vmio.h"
#include "int/interror.h"
#include "int/exec.h"

/*************************** IntSubroutineValue **************************/

IntSubroutineValue::IntSubroutineValue(IntBCORef bco)
    : bco(bco)
{ }

IntSubroutineValue::~IntSubroutineValue()
{ }

IntBCORef
IntSubroutineValue::getBytecodeObject()
{
    return bco;
}

void
IntSubroutineValue::call(IntExecutionContext& exc, IntDataSegment& args, bool want_result)
{
    exc.handleFunctionCall(bco, args, want_result);
}

bool
IntSubroutineValue::isProcedureCall()
{
    return bco->isProcedure();
}

int32_t
IntSubroutineValue::getDimension(int32_t /*which*/)
{
    return 0;
}

IntContext*
IntSubroutineValue::makeFirstContext()
{
    throw IntError::typeError(IntError::tyIterable);
}

string_t
IntSubroutineValue::toString(bool /*readable*/)
{
    string_t name = bco->getName();
    string_t result = "#<subr";
    if (name.size()) {
        result += ":";
        result += name;
    }
    return result + ">";
}

void
IntSubroutineValue::store(IntTagNode& tag, Stream& /*aux*/)
{
    IntVMSaveContext* vsc = IntVMSaveContext::getCurrentInstance();
    if (vsc != 0) {
        tag.tag   = IntTagNode::Tag_BCO;
        tag.value = vsc->addBCO(*bco);
    } else {
        throw IntError::notSerializable();
    }
}

IntSubroutineValue*
IntSubroutineValue::clone()
{
    return new IntSubroutineValue(bco);
}
