/**
  *  \file ui/widgets/numbersel.h
  *  \brief UINumberSelector widget
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UI_WIDGETS_NUMBERSEL_H
#define PCC_V2_UI_WIDGETS_NUMBERSEL_H

#include "ui/ui.h"
#include "cpluslib/sigslot.h"

/** Base class for a textual number selector. This handles up/down/+/-
    keys. Users must provide a drawContent() method to display the
    widget. */
class UINumberSelector : public UIWidget {
    long current;
    long low, high;
    int  step;
 public:
    UINumberSelector(int id, long low, long high, int step);
    ~UINumberSelector();
    void setValue(long val);
    long getValue() const { return current; }
    bool handleEvent(const UIEvent& e, bool second_pass);

    long getMin() const { return low; }
    long getMax() const { return high; }
    int  getStep() const { return step; }

    void setRange(int min, int max);

    void increment(int n);
    void decrement(int n);

    Signal1<int> sig_changed;

    bool doStandardDialog(string_t title, string_t prompt, string_t help);
};

#endif
